@@ -386,6 +386,10 @@ class ProgIter(_TQDMCompat, _BackwardsCompat):
         self.started = False
         self.finished = False
 
+        # indicates if the cursor is currently at the start of a line (True) or
+        # if characters have been written with no newline yet.
+        self._cursor_at_newline = True
+
         self._reset_internals()
 
     def __call__(self, iterable):
@@ -439,11 +443,14 @@ class ProgIter(_TQDMCompat, _BackwardsCompat):
         # Wrap input sequence in a generator
         for self._iter_idx, item in enumerate(self.iterable, start=self.initial + 1):
             yield item
-            if (self._iter_idx) % self.freq == 0:
-                # update progress information every so often
-                self._update_measurements()
-                self._update_estimates()
-                self.display_message()
+            self.step(0)  # inc is 0 because we already updated
+            # _between_idx = (self._iter_idx - self._last_idx)
+            # if (self._iter_idx) % self.freq == 0 or _between_idx > self.freq:
+            #     # update progress information every so often
+            #     # Note: this is the same logic as `self.step`, but inline
+            #     self._update_measurements()
+            #     self._update_estimates()
+            #     self.display_message()
         self.end()
 
     def step(self, inc=1):
@@ -472,9 +479,11 @@ class ProgIter(_TQDMCompat, _BackwardsCompat):
         if not self.enabled:
             return
         self._iter_idx += inc
-        self._update_measurements()
-        self._update_estimates()
-        self.display_message()
+        _between_idx = (self._iter_idx - self._now_idx)
+        if (self._iter_idx) % self.freq == 0 or _between_idx > self.freq:
+            self._update_measurements()
+            self._update_estimates()
+            self.display_message()
 
     def _reset_internals(self):
         """
